
#ifndef DIRVIEW_H
#define DIRVIEW_H

#include <qlistview.h>
#include <qfile.h>

class QTimer;
class QDragEnterEvent;
class QDragMoveEvent;
class QDragLeaveEvent;
class QDropEvent;

struct Point2D {
   float x, y;
   Point2D() : x(0), y(0) { }
   Point2D( float X, float Y ) : x( X ), y( Y ) { }
};

struct Rect2D {
   bool empty;
   float x, y, w, h;
   Rect2D() : empty( true ), x(0), y(0), w(0), h(0) { }
   Rect2D( float X, float Y, float W, float H )
      : empty( false ), x( X ), y( Y ), w( W ), h( H ) { }
   // Enlarge the rect to contain the given point
   void bound( const Point2D& p );
   // Sets this rect to be the smallest rect containing the
   // given rect (centred) and having the given aspect ratio.
   void set( const Rect2D& r, float desiredAspectRatio );
   // (X,Y) are coordinates of the pixel to convert, and (W,H) are
   // the dimensions (in pixels) of the viewport
   Point2D pixelToPoint( int X, int Y, int W, int H ) const;
};

class Directory : public QListViewItem {
private:
   QFile _file;
   Directory * _parentDir;
   bool _readable;
   bool _showDirsOnly;
   const QPixmap *_pixmap;

   Point2D _p;   // location within parent's 2D space

   bool _childrenHaveBeenMoved;
   Rect2D _r;  // 2D space for this directory
public:
   Directory( QListView * parent, const QString& filename );
   Directory(
      Directory * parent, const QString& filename, const QString &col2
   ) : QListViewItem( parent, filename, col2 ), _pixmap( 0 ),
      _childrenHaveBeenMoved( false ) { }
   Directory( Directory * parent, const QString& filename );
   void saveState();

   QString text( int column ) const;

   QString getName() const { return _file.name(); }
   QString fullName();

   void readContents();
   void setOpen( bool );
   void setup();

   const QPixmap *pixmap( int i ) const;
   void setPixmap( const QPixmap *p );

   const Point2D& getPoint() const { return _p; }
   void setPoint( const Point2D& p ) {
      _p = p;
      dynamic_cast< Directory* >( parent() )->setChildMovedFlag();
   }
   Rect2D& getRect() { return _r; }
   void setChildMovedFlag() { _childrenHaveBeenMoved = true; }
};

class FileItem : public QListViewItem {
   friend class Directory;
private:
   const QPixmap * _pixmap;

   Point2D _p;   // location within parent's 2D space
public:
   FileItem(
      QListViewItem *parent, const QString &s1,
      const QString &s2, const QString &s3
   ) : QListViewItem( parent, s1, s2, s3 ), _pixmap( 0 ) {}

   const QPixmap *pixmap( int i ) const;
   void setPixmap( const QPixmap *p );

   const Point2D& getPoint() const { return _p; }
   void setPoint( const Point2D& p ) {
      _p = p;
      dynamic_cast< Directory* >( parent() )->setChildMovedFlag();
   }
};

class DirectoryView : public QListView {
   Q_OBJECT
private:
   QApplication * _app;
   QString fullPath(QListViewItem* item);
   bool dirsOnly;
   QListViewItem *oldCurrent;
   QListViewItem *dropItem;
   QTimer* autoopen_timer;
   QPoint presspos;
   bool mousePressed;
   bool _showPixmaps;
public:
   DirectoryView( QWidget *parent = 0, const char *name = 0, bool sdo = FALSE );
   void setApp( QApplication * a ) { _app = a; }
   bool showDirsOnly() { return dirsOnly; }
   bool showPixmaps() const { return _showPixmaps; }
   void loadSubtree( const QString & s );
   void saveState() const;
public slots:
   //void setDir( const QString & );
   void toggleIcons();
   void loadParentOfCurrentSubtree();
   void loadSubtreeUnderCurrentItem();
   void computeSizeOfCurrentDirectory();
   void causeQuit();
signals:
   void folderSelected( const QString & );
   void contentsCleared();
protected slots:
   void slotFolderSelected( QListViewItem * );
   void openFolder();
protected:
   void contentsDragEnterEvent( QDragEnterEvent *e );
   void contentsDragMoveEvent( QDragMoveEvent *e );
   void contentsDragLeaveEvent( QDragLeaveEvent *e );
   void contentsDropEvent( QDropEvent *e );
   void contentsMouseMoveEvent( QMouseEvent *e );
   void contentsMousePressEvent( QMouseEvent *e );
   void contentsMouseReleaseEvent( QMouseEvent *e );
};

#endif /* DIRVIEW_H */

